# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

import wx
import wx.xrc

def get_child_controls(root, result=None):

    def append_sizer_controls(sizer, controls):
        for item in sizer.GetChildren():
            subsizer = item.GetSizer()
            if subsizer: append_sizer_controls(subsizer, controls)
            window = item.GetWindow()
            if window: get_controls(window, controls)
    
    if result is None: result = {}
    for child in root.GetChildren():
        if child.GetName():
            result[child.GetName()] = child
        get_controls(child, result)
    sizer = root.GetSizer()
    if sizer: append_sizer_controls(sizer, result)
    return result


_event_types = {wx.Button: wx.EVT_BUTTON, wx.Choice: wx.EVT_CHOICE, wx.CheckBox: wx.EVT_CHECKBOX}

def _default_control_event(control):
    for c in _event_types:
        if isinstance(control, c): return _event_types[c]
    return None

def control_for_binding(name, action, rootwindow, menubar=None):
    control = wx.xrc.XRCCTRL(rootwindow, name)
    if control:
        if action:
            event = _default_control_event(control)
            if event:
                control.Bind(event, action)
    elif menubar:
        control = menubar.FindItemById(wx.xrc.XRCID(name))
        if control:
            if action:
                rootwindow.Bind(wx.EVT_MENU, action, control)
    return control
    
def attach_control_bindings(bindings, rootwindow, menubar=None):
    """ Given a bindings list, root window, and optional menu bar, attempts to locate subwindows (controls)
        or menu items whose names are the first items of the bindings list. If the second item is nonempty, binds the "default" action
        of the control to the value, which should be a function object. Returns a dict whose keys are the names in
        the bindings dict and whose values are the associated controls. """
    controls = {}
    for key, action in bindings:
        if isinstance(key, (tuple,list)):
            controls[key[0]] = [control_for_binding(name, action, rootwindow, menubar) for name in key[1]]
        else:
            controls[key] = control_for_binding(key, action, rootwindow, menubar)
    return controls
    
    
# menus is ((menu1 name, ((item1 name, item1 label, item1 action)...)), (menu2 name...) )
# separator is '-', or '-?' for separator only on non-Mac (so special menus can be right)
def create_menubar(frame, menus):
    #ismac = 0 #('__WXMAC__'==wx.Platform) # wxPython does the right thing with separators?
    allitems = {}
    menubar = wx.MenuBar()
    #print menus
    for menu_name, menu_items in menus:
        menu = wx.Menu()
        menubar.Append(menu, menu_name)
        for itemdef in menu_items:
            #print itemdef
            if isinstance(itemdef, str):
                if itemdef.startswith('-'):
                    #nosep = ismac and itemdef.endswith('?')
                    #if not nosep:
                        menu.AppendSeparator()
            else:
                item_name, item_label, action = itemdef
                item_id = wx.xrc.XRCID(item_name) if isinstance(item_name, str) else item_name
                item = menu.Append(item_id, item_label)
                if action: frame.Bind(wx.EVT_MENU, action, item)
                allitems[item_name] = item
                
    frame.SetMenuBar(menubar)
    return allitems
    
def callafter_wrapper(func):
    """ Returns a wrapped function that will execute on the main thread using wx.CallAfter """
    return lambda *args, **kwargs: wx.CallAfter(lambda: func(*args, **kwargs))

import PGUtils
# get built-in json module in 2.6, or simplejson if available, or fake json object
json = PGUtils.import_json()

""" This class implements a dict wrapper for wx.Config. Getting and setting items are mapped to Config
Read and Write calls. All values are converted to JSON for storage in Config, and converted back when reading.
"""

class ConfigDict(dict):
    def __init__(self, wxconfig=None):
        if wxconfig is None:
            wxconfig = wx.Config()
        self.config = wxconfig
        
    def __getitem__(self, key):
        value = self.config.Read(key, '')
        #print 'get:',key,value
        if not value: raise KeyError(key)
        return json.loads(value)
        
    # need to override get as well as __getitem__?
    def get(self, key, defvalue=None):
        try:
            return self[key]
        except KeyError:
            return defvalue
        
    def __setitem__(self, key, value):
        #print 'set:',key,value
        self.config.Write(key, json.dumps(value))
        
    def __delitem__(self, key):
        self.config.DeleteEntry(key)
        
    def __contains__(self, key):
        return self.config.Exists(key)

    def commit(self):
        self.config.Flush()
        